;; Midge mode - for writing midge(1) source files

(require 'compile)

(defvar midge-mode-map nil
  "Local keymap for midge mode buffers.")

;; set up keymap
(if midge-mode-map nil
  (setq midge-mode-map (make-sparse-keymap))

  (define-key midge-mode-map "}" 'midge-close-bracket)
  (define-key midge-mode-map "\t" 'midge-indent-line)

  (define-key midge-mode-map "\C-c\C-ch" 'midge-head-block)
  (define-key midge-mode-map "\C-c\C-cg" 'midge-body-block)
  (define-key midge-mode-map "\C-c\C-cn" 'midge-channel-block)
  (define-key midge-mode-map "\C-c\C-cr" 'midge-repeat-block)
  (define-key midge-mode-map "\C-c\C-cb" 'midge-bend-block)
  (define-key midge-mode-map "\C-c\C-cd" 'midge-define-block)
  (define-key midge-mode-map "\C-c\C-cc" 'midge-choose-block)
  (define-key midge-mode-map "\C-c\C-ca" 'midge-chain-block)
  (define-key midge-mode-map "\C-c\C-ct" 'midge-tuplet-block)
  (define-key midge-mode-map "\C-c\C-dr" 'midge-repeat-line)
  (define-key midge-mode-map "\C-c\C-db" 'midge-bend-line)
  (define-key midge-mode-map "\C-c\C-dd" 'midge-define-line)
  (define-key midge-mode-map "\C-c\C-dc" 'midge-choose-line)
  (define-key midge-mode-map "\C-c\C-dt" 'midge-tuplet-line)

  (define-key midge-mode-map "\C-c\C-fp" 'midge-select-patch)
  (define-key midge-mode-map "\C-c\C-fd" 'midge-select-drum)
  (define-key midge-mode-map "\C-c\C-fs" 'midge-select-scale)

  (define-key midge-mode-map "\C-c\C-v\C-c" 'midge-compile)
  (define-key midge-mode-map "\C-c\C-v\C-b" 'midge-compile-background)
  (define-key midge-mode-map "\C-c\C-v\C-v" 'midge-compile-verbose)
  (define-key midge-mode-map "\C-c\C-v\C-d" 'midge-compile-debug)
  (define-key midge-mode-map "\C-c\C-v\C-f" 'midge-compile-ask)

  (define-key midge-mode-map "\C-c\C-v\C-p" 'midge-play-background)
  (define-key midge-mode-map "\C-c\C-v\C-l" 'midge-play-foreground)
  (define-key midge-mode-map "\C-c\C-v\C-o" 'midge-play-ask-background)
  (define-key midge-mode-map "\C-c\C-v\C-k" 'midge-play-ask-foreground))

(defvar midge-compiler "midge"
  "Path to the midge executable.")

(defvar midge-options nil
  "Options to pass to midge.")

(defvar midge-midi-player nil
  "Command to play midi files.")

(defvar midge-use-menus t
  "Whether to use the menus.")

(defconst midge-comment-regexp
  "#[^ \n]*" "regular expression for recognizing comments.")

(defconst midge-open-regexp
  "[^{}#\n]*{" "regular expression for opening bracket.")

(defconst midge-close-regexp
  "[^{}#\n]*}" "regular expression for closing bracket.")

(defconst midge-close-first-regexp
  "^[ \t]*}" 
  "regular expression to match a closing bracket as the
first non whitespace char.")

(defconst midge-whitespace-regexp
  "^[ \t]+" "regular expression to match whitespace.")

(defconst midge-blank-line-regexp
  "^[ \t]*$" "regular expression to match a blank line.")

(defconst midge-tuplet-ratio-regexp
  "^[0-9]+:[0-9]+$" "regular expression to match time ratio
in tuplet blocks.")

;; beep and print an error message
(defun midge-error-message (str)
  (beep)
  (message str)
  nil)

;; calculate indent level
(defun midge-indent-level ()
  "calculate the indent level of the current line."
  (let ((indent 0) (maxpos (line-beginning-position)))
    (goto-char (point-min))
    (while (< (point) maxpos)
      (cond ((looking-at midge-open-regexp)
	     (goto-char (match-end 0))
	     (setq indent (1+ indent)))
	    ((looking-at midge-close-regexp)
	     (goto-char (match-end 0))
	     (if (> indent 0)
		 (setq indent (1- indent))))
	    ((looking-at midge-comment-regexp)
	     (end-of-line))
	    ((eolp)
	     (if (< (point) maxpos)
		 (progn
		   (next-line 1)
		   (end-of-line)
		   (if (< (point) maxpos)
		       (beginning-of-line)))))
	    (t
	     (end-of-line))))
    indent))

(defun midge-indent-line ()
  "indent the current line."
  (interactive)
  (push-mark)
  (let ((indent (midge-indent-level)) (current 0) (safe-indent 0))
    (beginning-of-line)
    (if (looking-at midge-close-first-regexp)
	(if (> indent 0)
	    (setq indent (1- indent))))
    (setq safe-indent indent)
    (if (looking-at midge-whitespace-regexp)
	(setq current (- (match-end 0) (point))))
    (if (not (= current (* tab-width indent)))
	(progn
	  (delete-char current 't)
	  (while (> indent 0)
	    (insert "\t")
	    (setq indent (1- indent))))))
  (goto-char (mark))
  (if (looking-at midge-whitespace-regexp)
      (goto-char (match-end 0))))

(defun midge-close-bracket ()
  "insert a closing bracket, indenting if neccesary."
  (interactive)
  (push-mark)
  (beginning-of-line)
  (if (looking-at midge-blank-line-regexp)
      (progn
	(let ((indent 0))
	  (if (looking-at midge-whitespace-regexp)
	      (progn
		(goto-char (match-end 0))
		(if (>= (point) (mark))
		    (setq indent 1))))
	  (if (eolp)
	      (setq indent 1))
	  (if (= indent 1)
	      (progn
		(beginning-of-line)
		(if (looking-at midge-whitespace-regexp)
		    (progn
		      (kill-line)
		      (insert "\n")
		      (next-line -1)))
		(setq indent (1- (midge-indent-level)))
		(while (> indent 0)
		  (insert "\t")
		  (setq indent (1- indent)))))
	  (insert "}")))
    (goto-char (mark))
    (insert "}")))

(defun midge-head-block ()
  "Insert a head block."
  (interactive)
  (midge-indent-line)
  (insert "@head {\n\n")
  (midge-close-bracket)
  (insert "\n")
  (next-line -2)
  (midge-indent-line))

(defun midge-body-block ()
  "Insert a body block."
  (interactive)
  (midge-indent-line)
  (insert "@body {\n\n")
  (midge-close-bracket)
  (insert "\n")
  (next-line -2)
  (midge-indent-line))

(defun midge-channel-block (number name)
  "Insert a channel block, prompting for channel number
and instrument name, indenting and placing point apropriately."
  (interactive "nChannel number: \nsInstrument name: ")
  (if (or (< number 1) (> number 16))
      (midge-error (concat "Bad channel number: "
			   (number-to-string number)))
    (midge-indent-line)
    (if (equal name "")
	(insert (concat "@channel " (number-to-string number) " {\n\n"))
      (insert (concat "@channel " (number-to-string number)
		      " \"" name "\" {\n\n")))
    (midge-close-bracket)
    (insert "\n")
    (next-line -2)
    (midge-indent-line)))

(defun midge-repeat-block (count)
  "Insert a multiline repeat block, indenting as neccesary,
prompting for repeat count and placing point between the
brackets."
  (interactive "nRepeat count: ")
  (insert "%repeat ")
  (insert (number-to-string count))
  (midge-indent-line)
  (insert " {\n")
  (midge-indent-line)
  (insert "\n")
  (midge-close-bracket)
  (insert "\n")
  (next-line -1)
  (beginning-of-line)
  (backward-char 1))

(defun midge-choose-block (time)
  "Insert a multiline choose block, indenting as neccesary
and placing point between the brackets."
  (interactive "sRiff length: ")
  (insert "%choose ")
  (midge-indent-line)
  (if (equal time "")
      (insert "{\n")
    (insert (concat time " {\n")))
  (midge-indent-line)
  (insert "\n")
  (midge-close-bracket)
  (insert "\n")
  (next-line -1)
  (beginning-of-line)
  (backward-char 1))

(defun midge-chain-block (time)
  "Insert a chain block, indenting as neccesary
and placing point between the brackets."
  (interactive "sRiff length: ")
  (insert "%chain ")
  (midge-indent-line)
  (insert (concat time " {\n"))
  (midge-indent-line)
  (insert "\n")
  (midge-close-bracket)
  (insert "\n")
  (next-line -1)
  (beginning-of-line)
  (backward-char 1))

(defun midge-bend-block (note)
  "Insert a multiline bend block, indenting as neccesary,
prompting for initial note and placing point between the
brackets."
  (interactive "sInitial note: ")
  (insert "%bend ")
  (insert note)
  (midge-indent-line)
  (insert " {\n")
  (midge-indent-line)
  (insert "\n")
  (midge-close-bracket)
  (insert "\n")
  (next-line -1)
  (beginning-of-line)
  (backward-char 1))

(defun midge-define-block (name)
  "Insert a multiline define block, indenting as neccesary,
prompting for riff name and placing point between the
brackets."
  (interactive "sRiff name: ")
  (insert "%define ")
  (insert name)
  (midge-indent-line)
  (insert " {\n")
  (midge-indent-line)
  (insert "\n")
  (midge-close-bracket)
  (insert "\n")
  (next-line -1)
  (beginning-of-line)
  (backward-char 1))

(defun midge-tuplet-block (ratio)
  "Insert a multiline tuplet block, indenting as neccesary,
prompting for time ratio and placing point between the
brackets."
  (interactive "sTime ratio: ")
  (if (not (string-match midge-tuplet-ratio-regexp ratio))
      (midge-error-message
       (concat "bad time ratio `" ratio "\' (should be n:d)"))
    (insert "%tuplet ")
    (insert ratio)
    (midge-indent-line)
    (insert " {\n")
    (midge-indent-line)
    (insert "\n")
    (midge-close-bracket)
    (insert "\n")
    (next-line -1)
    (beginning-of-line)
    (backward-char 1)))

(defun midge-repeat-line (count)
  "Add a repeat block on a single line, prompting for
number of repeats and positioning point apropriately."
  (interactive "nRepeat count: ")
  (insert "%repeat ")
  (insert (number-to-string count))
  (insert " {  }\n")
  (backward-char 3))

(defun midge-bend-line (note)
  "Add a bend block on a single line, prompting for
initial note and positioning point apropriately."
  (interactive "sInitial note: ")
  (insert "%bend ")
  (insert note)
  (insert " {  }\n")
  (backward-char 3))

(defun midge-define-line (name)
  "Add a define block on a single line, prompting for
riff name and positioning point apropriately."
  (interactive "sName of riff: ")
  (insert "%define ")
  (insert name)
  (insert " {  }\n")
  (backward-char 3))

(defun midge-choose-line (time)
  "Add a choose block on a single line,
positioning point apropriately."
  (interactive "sRiff length: ")
  (if (equal time "")
      (insert "%choose {  }")
    (insert (concat "%choose " time " {   }\n")))
  (backward-char 3))

(defun midge-tuplet-line (ratio)
  "Add a tuplet block on a single line, prompting for
time ratio and positioning point apropriately."
  (interactive "sTime ratio: ")
  (if (not (string-match midge-tuplet-ratio-regexp ratio))
      (midge-error-message (concat "bad time ratio" ratio "(should be n:d)"))
    (insert "%tuplet ")
    (insert ratio)
    (insert " {  }\n")
    (backward-char 3)))

(defun midge-compile ()
  "Compile the current file."
  (interactive)
  (save-some-buffers)
  (let ((command (concat midge-compiler " " buffer-file-name)))
    (compile-internal command "No more errors")))

(defun midge-compile-background ()
  "Compile the current file."
  (interactive)
  (save-some-buffers)
  (let ((command (concat midge-compiler " " buffer-file-name "&")))
    (compile-internal command "No more errors")))

(defun midge-compile-debug ()
  "Compile the current file with debug option."
  (interactive)
  (save-some-buffers)
  (let ((command (concat midge-compiler " -d " buffer-file-name)))
    (compile-internal command "No more errors")))

(defun midge-compile-verbose ()
  "Compile the current file with verbose option."
  (interactive)
  (save-some-buffers)
  (let ((command (concat midge-compiler " -v " buffer-file-name)))
    (compile-internal command "No more errors")))

(defun midge-compile-ask (options)
  "Compile the current file with verbose option."
  (interactive "sOptions: ")
  (save-some-buffers)
  (let ((command (concat midge-compiler " " options " " buffer-file-name)))
    (compile-internal command "No more errors")))

(defun midge-play-background ()
  "play the midi file generated from the current midge source file."
  (interactive)
  (let ((filename (concat
		   (file-name-sans-extension buffer-file-name) ".mid")))
    (if (not (file-exists-p filename))
	 (midge-error-message "Current file has not been compiled")
      (shell-command (concat midge-midi-player " " filename "&")))))
	
(defun midge-play-foreground ()
  "play the midi file generated from the current midge source file."
  (interactive)
  (let ((filename (concat
		   (file-name-sans-extension buffer-file-name) ".mid")))
    (if (not (file-exists-p filename))
	(midge-error-message "Current file has not been compiled")
      (shell-command (concat midge-midi-player " " filename)))))
	
(defun midge-play-ask-background (file)
  "play the midi file chosen by the user in the background."
  (interactive "fMidi file: ")
  (let ((command (concat midge-midi-player " " file "&")))
    (shell-command command)))

(defun midge-play-ask-foreground (file)
  "play the midi file chosen by the user in the background."
  (interactive "fMidi file: ")
  (let ((command (concat midge-midi-player " " file)))
    (shell-command command)))

(defun midge-select-scale ()
  "Allow the user to choose a scale interactively."
  (interactive)
  (pop-to-buffer "*Select Scale*")
  (delete-other-windows)
  (insert "
0: Cancel
1: Major
2: Minor
3: Modes...
4: Jazz/Blues...
5: Misc...")
  (call-interactively 'midge-select-scale-family))

(defun midge-select-scale-family (family)
  (interactive "nEnter selection: ")
  (if (or (equal family 0) (> family 5))
      (kill-buffer "*Select Scale*"))
  (if (equal family 1)
      (progn
	(kill-buffer "*Select Scale*")
	(insert "major")))
  (if (equal family 2)
      (progn
	(kill-buffer "*Select Scale*")
	(insert "minor")))
  (if (equal family 3)
      (progn
	(kill-buffer "*Select Scale*")
	(midge-show-scale-modes)
	(call-interactively 'midge-select-scale-mode)))
  (if (equal family 4)
      (progn
	(kill-buffer "*Select Scale*")
	(midge-show-scale-jazz)
	(call-interactively 'midge-select-scale-jazz)))
  (if (equal family 5)
      (progn
	(kill-buffer "*Select Scale*")
	(midge-show-scale-misc)
	(call-interactively 'midge-select-scale-misc))))

(defun midge-show-scale-modes ()
  (pop-to-buffer "*Select Mode*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Ionian (major)
2: Dorian
3: Phrygian
4: Lydian
5: Mixolydian
6: Aeolian
7: Locrian
")))

(defun midge-show-scale-jazz ()
  (pop-to-buffer "*Select Jazz/Blues Scale*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Harmonic Minor
2: Jazz Minor (ascending melodic minor)
3: Bebop
4: Bebop Dorian
5: Bebop Mixolydian
6: Minor Pentatonic
7: Major Pentatonic
")))

(defun midge-show-scale-misc ()
  (pop-to-buffer "*Select Misc Scale*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Chromatic (half tone)
2: Whole Tone
3: Arabian
4: Spanish
5: Gypsy
")))

(defun midge-select-scale-mode (scale)
  (interactive "nEnter Selection: ")
  (if (or (equal scale 0) (> scale 7))
      (kill-buffer "*Select Mode*"))
  (if (equal scale 1)
      (progn
	(kill-buffer "*Select Mode*")
	(insert "ionian")))
  (if (equal scale 2)
      (progn
	(kill-buffer "*Select Mode*")
	(insert "dorian")))
  (if (equal scale 3)
      (progn
	(kill-buffer "*Select Mode*")
	(insert "phrygian")))
  (if (equal scale 4)
      (progn
	(kill-buffer "*Select Mode*")
	(insert "lydian")))
  (if (equal scale 5)
      (progn
	(kill-buffer "*Select Mode*")
	(insert "mixolydian")))
  (if (equal scale 6)
      (progn
	(kill-buffer "*Select Mode*")
	(insert "aeolian")))
  (if (equal scale 7)
      (progn
	(kill-buffer "*Select Mode*")
	(insert "locrian"))))

(defun midge-select-scale-jazz (scale)
  (interactive "nEnter Selection: ")
  (if (or (equal scale 0) (> scale 7))
      (kill-buffer "*Select Jazz/Blues Scale*"))
  (if (equal scale 1)
      (progn
	(kill-buffer "*Select Jazz/Blues Scale*")
	(insert "minor_harmonic")))
  (if (equal scale 2)
      (progn
	(kill-buffer "*Select Jazz/Blues Scale*")
	(insert "minor_jazz")))
  (if (equal scale 3)
      (progn
	(kill-buffer "*Select Jazz/Blues Scale*")
	(insert "bebop")))
  (if (equal scale 4)
      (progn
	(kill-buffer "*Select Jazz/Blues Scale*")
	(insert "bebop_dorian")))
  (if (equal scale 5)
      (progn
	(kill-buffer "*Select Jazz/Blues Scale*")
	(insert "bebop_mixolydian")))
  (if (equal scale 6)
      (progn
	(kill-buffer "*Select Jazz/Blues Scale*")
	(insert "minor_pentatonic")))
  (if (equal scale 7)
      (progn
	(kill-buffer "*Select Jazz/Blues Scale*")
	(insert "major_pentatonic"))))

(defun midge-select-scale-misc (scale)
  (interactive "nEnter Selection: ")
  (if (or (equal scale 0) (> scale 5))
      (kill-buffer "*Select Jazz/Blues Scale*"))
  (if (equal scale 1)
      (progn
	(kill-buffer "*Select Misc Scale*")
	(insert "chromatic")))
  (if (equal scale 2)
      (progn
	(kill-buffer "*Select Misc Scale*")
	(insert "whole_tone")))
  (if (equal scale 3)
      (progn
	(kill-buffer "*Select Misc Scale*")
	(insert "arabian")))
  (if (equal scale 4)
      (progn
	(kill-buffer "*Select Misc Scale*")
	(insert "spanish")))
  (if (equal scale 5)
      (progn
	(kill-buffer "*Select Misc Scale*")
	(insert "gypsy"))))

(defun midge-select-drum ()
  "Allow the user to choose a drum interactively."
  (interactive)
  (pop-to-buffer "*Select Drum*")
  (delete-other-windows)
  (insert "
0: Cancel
1: Bass & Snare Drums
2: Cymbals
3: Toms
4: Bongos & Congas etc.
5: Bells & Whistles etc.
6: Misc")
  (call-interactively 'midge-select-drum-family))

(defun midge-select-drum-family (family)
  (interactive "nEnter selection: ")
  (if (or (equal family 0) (> family 6))
      (kill-buffer "*Select Drum*"))
  (if (equal family 1)
      (progn
	(kill-buffer "*Select Drum*")
	(midge-show-bsdrum)
	(call-interactively 'midge-select-bsdrum)))
  (if (equal family 2)
      (progn
	(kill-buffer "*Select Drum*")
	(midge-show-cymbal)
	(call-interactively 'midge-select-cymbal)))
  (if (equal family 3)
      (progn
	(kill-buffer "*Select Drum*")
	(midge-show-tom)
	(call-interactively 'midge-select-tom)))
  (if (equal family 4)
      (progn
	(kill-buffer "*Select Drum*")
	(midge-show-bongcong)
	(call-interactively 'midge-select-bongcong)))
  (if (equal family 5)
      (progn
	(kill-buffer "*Select Drum*")
	(midge-show-bellwhst)
	(call-interactively 'midge-select-bellwhst)))
  (if (equal family 6)
      (progn
	(kill-buffer "*Select Drum*")
	(midge-show-miscdrum)
	(call-interactively 'midge-select-miscdrum))))

(defun midge-show-bsdrum ()
  (pop-to-buffer "*Select Bass or Snare Drum*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Acoustic Bass Drum
2: Bass Drum
3: Acoustic Snare
4: Electric Snare")))

(defun midge-show-cymbal ()
  (pop-to-buffer "*Select Cymbal*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Closed Hi-Hat
2: Pedal Hi-Hat
3: Open Hi-Hat
4: Crash Cymbal 1
5: Crash Cymbal 2
6: Ride Cymbal 1
7: Ride Cymbal 2
8: Chinese Cymbal
9: Splash Cymbal")))

(defun midge-show-tom ()
  (pop-to-buffer "*Select Tom*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Low Floor Tom
2: High Floor Tom
3: Low Tom
4: Low Mid Tom
5: High Mid Tom
6: High Tom")))

(defun midge-show-bongcong ()
  (pop-to-buffer "*Select Bongo*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: High Bongo
2: Low Bongo
3: Mute High Conga
4: Open High conga
5: Low Conga
6: High Timbale
7: Low Timbale")))

(defun midge-show-bellwhst ()
  (pop-to-buffer "*Select Bell or Whistle*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Ride Bell
2: Cowbell
3: High Agogo
4: Low Agogo
5: Muted Triangle
6: Open Triangle
7: Short Whistle
8: Long Whistle")))

(defun midge-show-miscdrum ()
  (pop-to-buffer "*Select Misc Drum*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0:  Cancel
1:  Side Stick
2:  Hand Clap
3:  Tambourine
4:  Vibraslap
5:  Cabasa
6:  Maracas
7:  Short Guiro
8:  Long Guiro
9:  Claves
10: High Wood Block
11: Low Wood Block
12: Muted Cuica
13: Open Cuica")))

(defun midge-select-bsdrum (bsdrum)
  (interactive "nEnter Selection: ")
  (if (or (equal bsdrum 0) (> bsdrum 4))
      (kill-buffer "*Select Bass or Snare Drum*"))
  (if (equal bsdrum 1)
      (progn
	(kill-buffer "*Select Bass or Snare Drum*")
	(insert "bd_ac")))
  (if (equal bsdrum 2)
      (progn
	(kill-buffer "*Select Bass or Snare Drum*")
	(insert "bd")))
  (if (equal bsdrum 3)
      (progn
	(kill-buffer "*Select Bass or Snare Drum*")
	(insert "sd_ac")))
  (if (equal bsdrum 4)
      (progn
	(kill-buffer "*Select Bass or Snare Drum*")
	(insert "sd_el"))))

(defun midge-select-cymbal (cymbal)
  (interactive "nEnter Selection: ")
  (if (or (equal cymbal 0) (> cymbal 9))
      (kill-buffer "*Select Cymbal*"))
  (if (equal cymbal 1)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "hh_c")))
  (if (equal cymbal 2)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "hh_p")))
  (if (equal cymbal 3)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "hh_o")))
  (if (equal cymbal 4)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "cym_crash")))
  (if (equal cymbal 5)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "cym_crash_2")))
  (if (equal cymbal 6)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "cym_ride")))
  (if (equal cymbal 7)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "cym_ride_2")))
  (if (equal cymbal 8)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "cym_chinese")))
  (if (equal cymbal 9)
      (progn
	(kill-buffer "*Select Cymbal*")
	(insert "cym_splash"))))

(defun midge-select-tom (tom)
  (interactive "nEnter Selection: ")
  (if (or (equal tom 0) (> tom 6))
      (kill-buffer "*Select Tom*"))
  (if (equal tom 1)
      (progn
	(kill-buffer "*Select Tom*")
	(insert "ftom_l")))
  (if (equal tom 2)
      (progn
	(kill-buffer "*Select Tom*")
	(insert "ftom_h")))
  (if (equal tom 3)
      (progn
	(kill-buffer "*Select Tom*")
	(insert "tom_l")))
  (if (equal tom 4)
      (progn
	(kill-buffer "*Select Tom*")
	(insert "tom_lm")))
  (if (equal tom 5)
      (progn
	(kill-buffer "*Select Tom*")
	(insert "tom_hm")))
  (if (equal tom 6)
      (progn
	(kill-buffer "*Select Tom*")
	(insert "tom_h"))))

(defun midge-select-bongcong (bongcong)
  (interactive "nEnter Selection: ")
  (if (or (equal bongcong 0) (> bongcong 7))
      (kill-buffer "*Select Bongo*"))
  (if (equal bongcong 1)
      (progn
	(kill-buffer "*Select Bongo*")
	(insert "bongo_h")))
  (if (equal bongcong 2)
      (progn
	(kill-buffer "*Select Bongo*")
	(insert "bongo_l")))
  (if (equal bongcong 3)
      (progn
	(kill-buffer "*Select Bongo*")
	(insert "conga_h_mute")))
  (if (equal bongcong 4)
      (progn
	(kill-buffer "*Select Bongo*")
	(insert "conga_h_open")))
  (if (equal bongcong 5)
      (progn
	(kill-buffer "*Select Bongo*")
	(insert "conga_l")))
  (if (equal bongcong 6)
      (progn
	(kill-buffer "*Select Bongo*")
	(insert "timbale_h")))
  (if (equal bongcong 7)
      (progn
	(kill-buffer "*Select Bongo*")
	(insert "timbale_l"))))

(defun midge-select-bellwhst (bellwhst)
  (interactive "nEnter Selection: ")
  (if (or (equal bellwhst 0) (> bellwhst 8))
      (kill-buffer "*Select Bell or Whistle*"))
  (if (equal bellwhst 1)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "ride_bell")))
  (if (equal bellwhst 2)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "cowbell")))
  (if (equal bellwhst 3)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "agogo_h")))
  (if (equal bellwhst 4)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "agogo_l")))
  (if (equal bellwhst 5)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "tri_mute")))
  (if (equal bellwhst 6)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "tri_open")))
  (if (equal bellwhst 7)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "whistle_sh")))
  (if (equal bellwhst 8)
      (progn
	(kill-buffer "*Select Bell or Whistle*")
	(insert "whistle_lg"))))

(defun midge-select-miscdrum (miscdrum)
  (interactive "nEnter Selection: ")
  (if (or (equal miscdrum 0) (> miscdrum 13))
      (kill-buffer "*Select Misc Drum*"))
  (if (equal miscdrum 1)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "stick")))
  (if (equal miscdrum 2)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "clap")))
  (if (equal miscdrum 3)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "tamb")))
  (if (equal miscdrum 4)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "vibraslap")))
  (if (equal miscdrum 5)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "cabasa")))
  (if (equal miscdrum 6)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "maracas")))
  (if (equal miscdrum 7)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "guiro_sh")))
  (if (equal miscdrum 8)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "guiro_lg")))
  (if (equal miscdrum 9)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "claves")))
  (if (equal miscdrum 10)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "wood_h")))
  (if (equal miscdrum 11)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "wood_l")))
  (if (equal miscdrum 12)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "cuica_mute")))
  (if (equal miscdrum 13)
      (progn
	(kill-buffer "*Select Misc Drum*")
	(insert "cuica_open"))))

(defun midge-select-patch ()
  "Allow the user to choose a patch interactively"
  (interactive)
  (pop-to-buffer "*Select Patch*")
  (delete-other-windows)
  (insert "
0:  Cancel
1:  Pianos
2:  Chromatic Percussion
3:  Organs
4:  Guitars
5:  Basses
6:  Solo Strings
7:  String Ensembles
8:  Brass
9:  Reed
10: Pipe
11: Synth Leads
12: Synth Pads
13: Synth Effects
14: Ethnic
15: Percussive
16: Sound Effects")
  (call-interactively 'midge-select-patch-family))

(defun midge-select-patch-family (family)
  (interactive "nEnter selection: ")
  (if (or (equal family 0) (> family 16))
      (kill-buffer "*Select Patch*"))
  (if (equal family 1)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-piano)
	(call-interactively 'midge-select-piano)))
  (if (equal family 2)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-chrom-perc)
	(call-interactively 'midge-select-chrom-perc)))
  (if (equal family 3)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-organ)
	(call-interactively 'midge-select-organ)))
  (if (equal family 4)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-guitar)
	(call-interactively 'midge-select-guitar)))
  (if (equal family 5)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-bass)
	(call-interactively 'midge-select-bass)))
  (if (equal family 6)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-str-solo)
	(call-interactively 'midge-select-str-solo)))
  (if (equal family 7)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-str-ens)
	(call-interactively 'midge-select-str-ens)))
  (if (equal family 8)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-brass)
	(call-interactively 'midge-select-brass)))
  (if (equal family 9)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-reed)
	(call-interactively 'midge-select-reed)))
  (if (equal family 10)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-pipe)
	(call-interactively 'midge-select-pipe)))
  (if (equal family 11)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-syn-lead)
	(call-interactively 'midge-select-syn-lead)))
  (if (equal family 12)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-syn-pad)
	(call-interactively 'midge-select-syn-pad)))
  (if (equal family 13)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-syn-fx)
	(call-interactively 'midge-select-syn-fx)))
  (if (equal family 14)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-ethnic)
	(call-interactively 'midge-select-ethnic)))
  (if (equal family 15)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-percussive)
	(call-interactively 'midge-select-percussive)))
  (if (equal family 16)
      (progn
	(kill-buffer "*Select Patch*")
	(midge-show-sound-fx)
	(call-interactively 'midge-select-sound-fx))))
  
(defun midge-show-piano ()
  (pop-to-buffer "*Select Piano*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Acoustic Grand Piano
2: Bright Acoustic Piano
3: Electric Grand Piano
4: Honky Tonk Piano
5: Electric Piano 1
6: Electric Piano 2
7: Harpsichord
8: Clavinet")))

(defun midge-show-chrom-perc ()
  (pop-to-buffer "*Select Chromatic Percussion*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Celesta
2: Glockenspiel 
3: Music Box    
4: Vibraphone   
5: Marimba      
6: Xylophone
7: Tubular Bells
8: Dulcimer")))

(defun midge-show-organ ()
  (pop-to-buffer "*Select Organ*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Drawbar Organ   
2: Percussive Organ
3: Rock Organ      
4: Church Organ    
5: Reed Organ      
6: Accoridan       
7: Harmonica       
8: Tango Accordian 
")))

(defun midge-show-guitar ()
  (pop-to-buffer "*Select Guitar*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Nylon String Guitar  
2: Steel String Guitar  
3: Electric Jazz Guitar 
4: Electric Clean Guitar
5: Electric Muted Guitar
6: Overdriven Guitar    
7: Distortion Guitar    
8: Guitar Harmonics     
")))

(defun midge-show-bass ()
  (pop-to-buffer "*Select Bass*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Acoustic Bass        
2: Electric Bass (finger)
3: Electric Bass (pick)  
4: Fretless Bass        
5: Slap Bass 1          
6: Slap Bass 2          
7: Synth Bass 1         
8: Synth Bass 2         
")))

(defun midge-show-str-solo ()
  (pop-to-buffer "*Select Solo Strings*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Violin            
2: Viola             
3: Cello             
4: Contrabass        
5: Tremolo Strings   
6: Pizzicato Strings 
7: Orchestral Strings
8: Timpani")))

(defun midge-show-str-ens ()
  (pop-to-buffer "*Select String Ensemble*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: String Ensemble 1
2: String Ensemble 2
3: SynthStrings 1   
4: SynthStrings 2   
5: Choir Aahs       
6: Voice Oohs       
7: Synth Voice      
8: Orchestra Hit")))

(defun midge-show-brass ()
  (pop-to-buffer "*Select Brass*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Trumpet      
2: Trombone     
3: Tuba         
4: Muted Trumpet
5: French Horn  
6: Brass Section
7: SynthBrass 1 
8: SynthBrass 2")))

(defun midge-show-reed ()
  (pop-to-buffer "*Select Reed*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Soprano Sax 
2: Alto Sax    
3: Tenor Sax   
4: Baritone Sax
5: Oboe        
6: English Horn
7: Bassoon     
8: Clarinet")))

(defun midge-show-pipe ()
  (pop-to-buffer "*Select Pipe*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Piccolo     
2: Flute       
3: Recorder    
4: Pan Flute   
5: Blown Bottle
6: Skakuhachi  
7: Whistle     
8: Ocarina")))

(defun midge-show-syn-lead ()
  (pop-to-buffer "*Select Synth Lead*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Lead 1 (square)   
2: Lead 2 (sawtooth) 
3: Lead 3 (calliope) 
4: Lead 4 (chiff)    
5: Lead 5 (charang)  
6: Lead 6 (voice)    
7: Lead 7 (fifths)   
8: Lead 8 (bass+lead)")))

(defun midge-show-syn-pad ()
  (pop-to-buffer "*Select Synth Pad*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Pad 1 (new age)  
2: Pad 2 (warm)     
3: Pad 3 (polysynth)
4: Pad 4 (choir)    
5: Pad 5 (bowed)    
6: Pad 6 (metallic) 
7: Pad 7 (halo)     
8: Pad 8 (sweep)")))

(defun midge-show-syn-fx ()
  (pop-to-buffer "*Select Synth Effects*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: FX 1 (rain)      
2: FX 2 (soundtrack)
3: FX 3 (crystal)   
4: FX 4 (atmosphere)
5: FX 5 (brightness)
6: FX 6 (goblins)   
7: FX 7 (echoes)    
8: FX 8 (sci-fi)")))

(defun midge-show-ethnic ()
  (pop-to-buffer "*Select Ethnic*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Sitar   
2: Banjo   
3: Shamisen
4: Koto    
5: Kalimba 
6: Bagpipe 
7: Fiddle  
8: Shanai")))

(defun midge-show-percussive ()
  (pop-to-buffer "*Select Percussive*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Tinkle Bell   
2: Agogo         
3: Steel Drums   
4: Woodblock     
5: Taiko Drum    
6: Melodic Tom   
7: Synth Drum    
8: Reverse Cymbal")))

(defun midge-show-sound-fx ()
  (pop-to-buffer "*Select Sound Effects*" nil)
  (delete-other-windows)
  (if (looking-at midge-blank-line-regexp)
      (insert "
0: Cancel
1: Guitar Fret Noise
2: Breath Noise     
3: Seashore         
4: Bird Tweet       
5: Telephone Ring   
6: Helicopter       
7: Applause         
8: Gunshot")))

(defun midge-select-piano (piano)
  (interactive "nEnter Selection: ")
  (if (or (equal piano 0) (> piano 8))
      (kill-buffer "*Select Piano*"))
  (if (equal piano 1)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "piano_grand_ac")))
  (if (equal piano 2)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "piano_br")))
  (if (equal piano 3)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "piano_grand_el")))
  (if (equal piano 4)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "piano_ht")))
  (if (equal piano 5)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "piano_el_1")))
  (if (equal piano 6)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "piano_el_2")))
  (if (equal piano 7)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "harpsichord")))
  (if (equal piano 8)
      (progn
	(kill-buffer "*Select Piano*")
	(insert "clavinet"))))

(defun midge-select-chrom-perc (chrom-perc)
  (interactive "nEnter Selection: ")
  (if (or (equal chrom-perc 0) (> chrom-perc 8))
      (kill-buffer "*Select Chromatic Percussion*"))
  (if (equal chrom-perc 1)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "celesta")))
  (if (equal chrom-perc 2)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "glockenspiel")))
  (if (equal chrom-perc 3)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "music_box")))
  (if (equal chrom-perc 4)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "vibraphone")))
  (if (equal chrom-perc 5)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "marimba")))
  (if (equal chrom-perc 6)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "xylophone")))
  (if (equal chrom-perc 7)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "tubalar_bells")))
  (if (equal chrom-perc 8)
      (progn
	(kill-buffer "*Select Chromatic Percussion*")
	(insert "dulcimer"))))

(defun midge-select-organ (organ)
  (interactive "nEnter Selection: ")
  (if (or (equal organ 0) (> organ 8))
      (kill-buffer "*Select Organ*"))
  (if (equal organ 1)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "organ_dbar")))
  (if (equal organ 2)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "organ_perc")))
  (if (equal organ 3)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "organ_rock")))
  (if (equal organ 4)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "organ_church")))
  (if (equal organ 5)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "organ_reed")))
  (if (equal organ 6)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "accordian")))
  (if (equal organ 7)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "harmonica")))
  (if (equal organ 8)
      (progn
	(kill-buffer "*Select Organ*")
	(insert "accordian_tango"))))

(defun midge-select-guitar (guitar)
  (interactive "nEnter Selection: ")
  (if (or (equal guitar 0) (> guitar 8))
      (kill-buffer "*Select Guitar*"))
  (if (equal guitar 1)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_nylon")))
  (if (equal guitar 2)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_steel")))
  (if (equal guitar 3)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_jazz")))
  (if (equal guitar 4)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_clean")))
  (if (equal guitar 5)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_muted")))
  (if (equal guitar 6)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_od")))
  (if (equal guitar 7)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_dist")))
  (if (equal guitar 8)
      (progn
	(kill-buffer "*Select Guitar*")
	(insert "guitar_harm"))))

(defun midge-select-bass (bass)
  (interactive "nEnter Selection: ")
  (if (or (equal bass 0) (> bass 8))
      (kill-buffer "*Select Bass*"))
  (if (equal bass 1)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_ac")))
  (if (equal bass 2)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_fg")))
  (if (equal bass 3)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_pick")))
  (if (equal bass 4)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_fless")))
  (if (equal bass 5)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_slap_1")))
  (if (equal bass 6)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_slap_2")))
  (if (equal bass 7)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_syn_1")))
  (if (equal bass 8)
      (progn
	(kill-buffer "*Select Bass*")
	(insert "bass_syn_2"))))

(defun midge-select-str-solo (solo-str)
  (interactive "nEnter Selection: ")
  (if (or (equal solo-str 0) (> solo-str 8))
      (kill-buffer "*Select Solo Strings*"))
  (if (equal solo-str 1)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "violin")))
  (if (equal solo-str 2)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "viola")))
  (if (equal solo-str 3)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "cello")))
  (if (equal solo-str 4)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "contrabass")))
  (if (equal solo-str 5)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "str_trem")))
  (if (equal solo-str 6)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "str_pizz")))
  (if (equal solo-str 7)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "str_orch")))
  (if (equal solo-str 8)
      (progn
	(kill-buffer "*Select Solo Strings*")
	(insert "timpani"))))

(defun midge-select-str-ens (ensemble)
  (interactive "nEnter Selection: ")
  (if (or (equal ensemble 0) (> ensemble 8))
      (kill-buffer "*Select String Ensemble*"))
  (if (equal ensemble 1)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "str_ens_1")))
  (if (equal ensemble 2)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "str_ens_2")))
  (if (equal ensemble 3)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "str_syn_1")))
  (if (equal ensemble 4)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "str_syn_2")))
  (if (equal ensemble 5)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "choir_aahs")))
  (if (equal ensemble 6)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "voice_oohs")))
  (if (equal ensemble 7)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "voice_syn")))
  (if (equal ensemble 8)
      (progn
	(kill-buffer "*Select String Ensemble*")
	(insert "orch_hit"))))

(defun midge-select-brass (brass)
  (interactive "nEnter Selection: ")
  (if (or (equal brass 0) (> brass 8))
      (kill-buffer "*Select Brass*"))
  (if (equal brass 1)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "trumpet")))
  (if (equal brass 2)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "trombone")))
  (if (equal brass 3)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "tuba")))
  (if (equal brass 4)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "trumpet_muted")))
  (if (equal brass 5)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "horn_fr")))
  (if (equal brass 6)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "brass")))
  (if (equal brass 7)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "brass_syn_1")))
  (if (equal brass 8)
      (progn
	(kill-buffer "*Select Brass*")
	(insert "brass_syn_2"))))

(defun midge-select-reed (reed)
  (interactive "nEnter Selection: ")
  (if (or (equal reed 0) (> reed 8))
      (kill-buffer "*Select Reed*"))
  (if (equal reed 1)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "sax_sop")))
  (if (equal reed 2)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "sax_alt")))
  (if (equal reed 3)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "sax_ten")))
  (if (equal reed 4)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "sax_bar")))
  (if (equal reed 5)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "oboe")))
  (if (equal reed 6)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "horn_en")))
  (if (equal reed 7)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "bassoon")))
  (if (equal reed 8)
      (progn
	(kill-buffer "*Select Reed*")
	(insert "clarinet"))))

(defun midge-select-pipe (pipe)
  (interactive "nEnter Selection: ")
  (if (or (equal pipe 0) (> pipe 8))
      (kill-buffer "*Select Pipe*"))
  (if (equal pipe 1)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "piccolo")))
  (if (equal pipe 2)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "flute")))
  (if (equal pipe 3)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "recorder")))
  (if (equal pipe 4)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "flute_pan")))
  (if (equal pipe 5)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "bottle")))
  (if (equal pipe 6)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "skakuhachi")))
  (if (equal pipe 7)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "whistle")))
  (if (equal pipe 8)
      (progn
	(kill-buffer "*Select Pipe*")
	(insert "ocarina"))))

(defun midge-select-syn-lead (lead)
  (interactive "nEnter Selection: ")
  (if (or (equal lead 0) (> lead 8))
      (kill-buffer "*Select Synth Lead*"))
  (if (equal lead 1)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_sq")))
  (if (equal lead 2)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_saw")))
  (if (equal lead 3)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_calliope")))
  (if (equal lead 4)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_chiff")))
  (if (equal lead 5)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_charang")))
  (if (equal lead 6)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_voice")))
  (if (equal lead 7)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_fifth")))
  (if (equal lead 8)
      (progn
	(kill-buffer "*Select Synth Lead*")
	(insert "lead_basslead"))))

(defun midge-select-syn-pad (pad)
  (interactive "nEnter Selection: ")
  (if (or (equal pad 0) (> pad 8))
      (kill-buffer "*Select Synth Pad*"))
  (if (equal pad 1)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "pad_new_age")))
  (if (equal pad 2)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "pad_warm")))
  (if (equal pad 3)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "polysynth")))
  (if (equal pad 4)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "pad_choir")))
  (if (equal pad 5)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "pad_bowed")))
  (if (equal pad 6)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "pad_metal")))
  (if (equal pad 7)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "pad_halo")))
  (if (equal pad 8)
      (progn
	(kill-buffer "*Select Synth Pad*")
	(insert "pad_sweep"))))

(defun midge-select-syn-fx (synth-fx)
  (interactive "nEnter Selection: ")
  (if (or (equal synth-fx 0) (> synth-fx 8))
      (kill-buffer "*Select Synth Effects*"))
  (if (equal synth-fx 1)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_rain")))
  (if (equal synth-fx 2)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_strack")))
  (if (equal synth-fx 3)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_crystal")))
  (if (equal synth-fx 4)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_atmos")))
  (if (equal synth-fx 5)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_bright")))
  (if (equal synth-fx 6)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_goblin")))
  (if (equal synth-fx 7)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_echo")))
  (if (equal synth-fx 8)
      (progn
	(kill-buffer "*Select Synth Effects*")
	(insert "fx_scifi"))))

(defun midge-select-ethnic (ethnic)
  (interactive "nEnter Selection: ")
  (if (or (equal ethnic 0) (> ethnic 8))
      (kill-buffer "*Select Ethnic*"))
  (if (equal ethnic 1)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "sitar")))
  (if (equal ethnic 2)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "banjo")))
  (if (equal ethnic 3)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "shamisen")))
  (if (equal ethnic 4)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "koto")))
  (if (equal ethnic 5)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "kalimba")))
  (if (equal ethnic 6)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "bagpipe")))
  (if (equal ethnic 7)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "fiddle")))
  (if (equal ethnic 8)
      (progn
	(kill-buffer "*Select Ethnic*")
	(insert "shanai"))))

(defun midge-select-percussive (percussive)
  (interactive "nEnter Selection: ")
  (if (or (equal percussive 0) (> percussive 8))
      (kill-buffer "*Select Percussive*"))
  (if (equal percussive 1)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "bell_tinkle")))
  (if (equal percussive 2)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "agogo")))
  (if (equal percussive 3)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "drum_steel")))
  (if (equal percussive 4)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "woodblock")))
  (if (equal percussive 5)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "drum_taiko")))
  (if (equal percussive 6)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "tom_melodic")))
  (if (equal percussive 7)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "drum_syn")))
  (if (equal percussive 8)
      (progn
	(kill-buffer "*Select Percussive*")
	(insert "cymbal_rev"))))

(defun midge-select-sound-fx (sound-fx)
  (interactive "nEnter Selection: ")
  (if (or (equal sound-fx 0) (> sound-fx 8))
      (kill-buffer "*Select Sound Effects*"))
  (if (equal sound-fx 1)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_fret")))
  (if (equal sound-fx 2)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_breath")))
  (if (equal sound-fx 3)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_sea")))
  (if (equal sound-fx 4)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_tweet")))
  (if (equal sound-fx 5)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_phone")))
  (if (equal sound-fx 6)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_copter")))
  (if (equal sound-fx 7)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_applause")))
  (if (equal sound-fx 8)
      (progn
	(kill-buffer "*Select Sound Effects*")
	(insert "fx_gun"))))

(defvar midge-menu
  (list "Midge"
	(list "Block..."
	      ["head"      midge-head-block t]
	      ["body"      midge-body-block t]
	      ["define"    midge-define-block t]
	      ["channel"   midge-channel-block t]
	      ["repeat"    midge-repeat-block t]
	      ["bend"      midge-bend-block t]
	      ["choose"    midge-choose-block t]
	      ["chain"     midge-chain-block t]
	      ["tuplet"    midge-tuplet-block t]
	      )
	(list "Line..."
	      ["repeat"    midge-repeat-line t]
	      ["bend"      midge-bend-line t]
	      ["choose"    midge-choose-line t]
	      ["chain"     midge-chain-line t]
	      ["tuplet"    midge-tuplet-line t]
	      )
	"-"
	["Drum"    midge-select-drum t]
	["Patch"   midge-select-patch t]
	["Scale"   midge-select-scale t]
	"-"
	(list "Compile..."
	      ["Simple"          midge-compile t]
	      ["Verbose"         midge-compile-verbose t]
	      ["Debug"           midge-compile-debug t]
	      ["Prompt for args" midge-compile-ask t]
	      )
	(list "Play..."
	      ["Current foreground" midge-play-foreground t]
	      ["Current background" midge-play-background t]
	      ["Prompt foreground"  midge-play-ask-foreground t]
	      ["Prompt background"  midge-play-ask-background t]
	      )))

(if midge-use-menus
    (progn
      (require 'easymenu)
      (easy-menu-do-define 'midge-menu1
			   midge-mode-map
			   "Menu for midge-mode."
			   midge-menu)))

(defun midge-mode ()
  "Midge mode, for writing midge(1) source files."
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'midge-mode)
  (setq mode-name "Midge")
  (use-local-map midge-mode-map)
  (setq tab-width 4))
